Needs["Combinatorica`"]

Needs["GraphUtilities`"]

Clear[EdgesWithVertices];
EdgesWithVertices[v_, e_] := 
 Module[{i, j, n = Length[e], m = Length[v], edges = {}},
  For[i = 1, i <= n, i++,
   For[j = 1, j <= m, j++,
     If[MemberQ[e[[i]], v[[j]]], AppendTo[edges, e[[i]]]; 
       Break[]];];]; edges]

Clear[SubGraph];
SubGraph[graph_, v_] := 
 Combinatorica`Graph[EdgesWithVertices[v, Edges[graph]], v]

Clear[BivariatesPolynom1];
BivariatesPolynom1[graph_, x_, y_] := 
 x^Length[Vertices[graph]] /; Length[Edges[graph]] == 0
BivariatesPolynom1[graph_, x_, y_] := 
 Apply[Times, 
   Map[BivariatesPolynom1[SubGraph[graph, #], x, y] &, 
    Combinatorica`ConnectedComponents[graph]]] /; 
  Not[Combinatorica`ConnectedGraphQ[graph]]
BivariatesPolynom1[graph_, x_, y_] := Module[{e, e1, e2, g1, g2, g3},
   e = Edges[graph][[1]];
   g1 = DeleteEdge[graph, e];
   g2 = MakeSimple[Contract[graph, e]];
   e1 = Min[e];
   e2 = Max[e];
   g3 = DeleteVertices[graph, {e1, e2}];
   Return[
    Expand[BivariatesPolynom1[g1, x, y] - 
      BivariatesPolynom1[g2, x, 
       y] + (x - y) BivariatesPolynom1[g3, x, y]]];] /; 
  Length[Edges[graph]] > 0
$RecursionLimit = Infinity

Clear[BivariatesPolynom2];
BivariatesPolynom2[graph_, x_, y_] :=
 Sum[Binomial[Length[Vertices[graph]], k]*(x - y)^k*
    FunctionExpand[
     FactorialPower[y, Length[Vertices[graph]] - k]], {k, 0, 
    Length[Vertices[graph]]}] /; 
  Length[Edges[graph]] == 
   Length[Vertices[graph]]*(Length[Vertices[graph]] - 1)/2
BivariatesPolynom2[graph_, x_, y_] := 
 Apply[Times, 
   Map[BivariatesPolynom2[SubGraph[graph, #], x, y] &, 
    Combinatorica`ConnectedComponents[graph]]] /; 
  Not[Combinatorica`ConnectedGraphQ[graph]]
BivariatesPolynom2[graph_, x_, y_] := Module[{e, e1, e2, g1, g2, g3},
   e = Complement[Subsets[VertexList[graph], {2}], EdgeList[graph]][[
     1]];
   g1 = AddEdge[graph, e];
   g2 = MakeSimple[Contract[AddEdge[graph, e], e]];
   e1 = Min[e];
   e2 = Max[e];
   g3 = DeleteVertices[AddEdge[graph, e], {e1, e2}];
   Return[
    Expand[BivariatesPolynom2[g1, x, y] + 
      BivariatesPolynom2[g2, x, y] - (x - y)*
       BivariatesPolynom2[g3, x, y]]];] /; 
  Length[Edges[graph]] < 
   Length[Vertices[graph]]*(Length[Vertices[graph]] - 1)/2
$RecursionLimit = Infinity

Clear[IndependentSetPartitions]
IndependentSetPartitions[graph_] := 
 Module[{i, j, n = Length[SetPartitions[VertexList[graph]]], 
   sepa = SetPartitions[VertexList[graph]], 
   IndependentPartitions = {}},
  For[i = 1, i <= n, i++,
   append = True;
   For[j = 1, j <= Length[sepa[[i]]], j++,
    If[Not[IndependentSetQ[graph, sepa[[i]][[j]]]], append = False; 
      Break[]];];
   If[append, AppendTo[IndependentPartitions, sepa[[i]]]];]; 
  IndependentPartitions]

Clear[BivariatesPolynom3]
BivariatesPolynom3[graph_, x_, y_] :=
 Sum[(x - y)^(Length[VertexList[graph]] - i)*
   Sum[Sum[FunctionExpand[
      FactorialPower[y, 
       Length[IndependentSetPartitions[
          InduceSubgraph[graph, 
           Subsets[VertexList[graph], {i}][[j]]]][[k]]]]], {k, 1, 
      Length [IndependentSetPartitions[
        InduceSubgraph[graph, 
         Subsets[VertexList[graph], {i}][[j]]]]]}], {j, 1, 
     Length[Subsets[VertexList[graph], {i}]]}], {i, 0, 
   Length[VertexList[graph]]}]

Clear[IndependentSetPartitionNumberSingletons]
IndependentSetPartitionNumberSingletons[graph_,a_]:=Module[{j,n=Length[IndependentSetPartitions[graph][[a]]],isep=IndependentSetPartitions[graph][[a]],SingletonNumberIndependentPartition={}},
For[j=1,j<=n,j++,
If[Length[isep[[j]]]==1,AppendTo[SingletonNumberIndependentPartition,isep[[j]]]];
];
Length[SingletonNumberIndependentPartition]
]

Clear[BivariatesPolynom4]
BivariatesPolynom4[graph_,x_,y_]:=Expand[
Sum[Sum[Binomial[IndependentSetPartitionNumberSingletons[graph,k],l]*(x-y)^l*FunctionExpand[FactorialPower[y,Length [IndependentSetPartitions[graph][[k]]]-l]],{l,0,IndependentSetPartitionNumberSingletons[graph,k]}],{k,1,Length[IndependentSetPartitions[graph]]}]
]

Clear[BivariatesPolynomKs];
BivariatesPolynomKs[s_,x_,y_]:=Sum[Binomial[s,k]*(x-y)^k*FunctionExpand[FactorialPower[y,s-k]],{k,0,s}]

Clear[BivariatesPolynom5];
BivariatesPolynom5[s_,r_,x_,y_]:=(x-y)*BivariatesPolynomKs[s,x,y]+y*(BivariatesPolynomKs[s,x-1,y-1]+(s-r)*BivariatesPolynomKs[s-1,x-1,y-1])

Clear[BivariatesPolynom6]
BivariatesPolynom6[s_,k_,x_,y_]:=Sum[Binomial[s,i]*(x-y)^{s-i}*FunctionExpand[FactorialPower[y,i]]*(x-i)^{k},{i,0,s}]
BivariatesPolynom6[1,s_,k_,x_,y_]:=BivariatesPolynom6[s,k,x,y]+BivariatesPolynom6[s,k-1,x,y]-(x-y)*BivariatesPolynom6[s-1,k-1,x,y]
BivariatesPolynom6[t_,s_,k_,x_,y_]:=BivariatesPolynom6[t-1,s,k,x,y]+BivariatesPolynom6[t-1,s,k-1,x,y]-(x-y)*BivariatesPolynom6[t-1,s-1,k-1,x,y]

Clear[UnivariatesPolynom6]
UnivariatesPolynom6[s_,k_,y_]:=FunctionExpand[FactorialPower[y,s]]*(y-s)^{k}
UnivariatesPolynom6[1,s_,k_,y_]:=UnivariatesPolynom6[s,k,y]+UnivariatesPolynom6[s,k-1,y]
UnivariatesPolynom6[t_,s_,k_,y_]:=UnivariatesPolynom6[t-1,s,k,y]+UnivariatesPolynom6[t-1,s,k-1,y]

Clear[BivariatesPolynom7];
BivariatesPolynom7[r_,s_,x_,y_]:=Sum[Binomial[r,k]*(x-y)^(r-k)*Sum[StirlingS2[k,j]*FunctionExpand[FactorialPower[y,j]]*(x-j)^s,{j,0,k}],{k,0,r}]
BivariatesPolynom7[1,r_,s_,x_,y_]:=BivariatesPolynom7[r,s,x,y]+y*BivariatesPolynom7[r-1,s-1,x-1,y-1]
BivariatesPolynom7[n_,r_,s_,x_,y_]:=BivariatesPolynom7[n-1,r,s,x,y]+y*BivariatesPolynom7[n-1,r-1,s-1,x-1,y-1]

Clear[UnivariatesPolynom7];
UnivariatesPolynom7[r_,s_,y_]:=Sum[StirlingS2[r,j]*FunctionExpand[FactorialPower[y,j]]*(y-j)^s,{j,0,r}]
UnivariatesPolynom7[1,r_,s_,y_]:=UnivariatesPolynom7[r,s,y]+y*UnivariatesPolynom7[r-1,s-1,y-1]
UnivariatesPolynom7[n_,r_,s_,y_]:=UnivariatesPolynom7[n-1,r,s,y]+y*UnivariatesPolynom7[n-1,r-1,s-1,y-1]

Clear[BivariatesPolynom8]
BivariatesPolynom8[r_,1,x_,y_]:=x^r
BivariatesPolynom8[r_,k_,x_,y_]:=Sum[Binomial[r,i]*(x-y)^i*Sum[StirlingS2[r-i,j]*FunctionExpand[FactorialPower[y,j]]*BivariatesPolynom8[r,k-1,x-j,y-j],{j,0,r-i}],{i,0,r}]

Clear[UnivariatesPolynom8]
UnivariatesPolynom8[r_,1,y_]:=y^r
UnivariatesPolynom8[r_,k_,y_]:=Sum[StirlingS2[r,j]*FunctionExpand[FactorialPower[y,j]]*UnivariatesPolynom8[r,k-1,y-j],{j,0,r}]

Clear[SubsetsWithRemainingEdges]
SubsetsWithRemainingEdges[graph_,m_]:=Module[{i,n=Length[Subsets[VertexList[graph]]],ver=VertexList[graph],sub=Subsets[VertexList[graph]],RemainingSubsets={}},
For[i=1, i<=n,i++,
If[Length[Edges[InduceSubgraph[graph,Complement[ver,sub[[i]]]]]]>= m,
AppendTo[RemainingSubsets,sub[[i]]]]];
RemainingSubsets]

Clear[ConnectedSetPartitions]
ConnectedSetPartitions[graph_]:=Module[{i,j,n=Length[SetPartitions[VertexList[graph]]],part=SetPartitions[VertexList[graph]],ConnectedPartitions={}},
For[i=1, i<=n,i++,
append = True;
For[j=1,j<=Length[part[[i]]],j++,
If[Not[ConnectedQ[InduceSubgraph[graph,part[[i]][[j]]]]],append=False;Break[]];];
If[append,AppendTo[ConnectedPartitions,part[[i]]]];];
ConnectedPartitions]

Clear[Pairs];
Pairs[p_]:=Flatten[Map[Subsets[#,{2}]&,p],1]

Clear[PartitionsOf];
PartitionsOf[graph_,m_]:=Select[ConnectedSetPartitions[graph],Length[Intersection[Pairs[#],Edges[graph]]]==m&]

Clear[IndependentPartitionsOfPartitionsOf]
IndependentPartitionsOfPartitionsOf[graph_,m_,p_]:=
Module[{i,j,n=Length[SetPartitions[PartitionsOf[graph,m][[p]]]],part=SetPartitions[PartitionsOf[graph,m][[p]]],IndependentPartitions={}},
For[i=1, i<=n,i++,
append=True;
For[j=1,j<= Length[part[[i]]],j++,
If[Length[part[[i]][[j]]]!= Length[Combinatorica`ConnectedComponents[InduceSubgraph[graph,Flatten[part[[i]][[j]]]]]],append=False;Break[]];];
If[append,AppendTo[IndependentPartitions,part[[i]]];];];
IndependentPartitions]

Clear[Koeffizient1]
Koeffizient1[graph_,m_,x_,y_]:=
Module[{suwir=SubsetsWithRemainingEdges[graph,m]},
Sum[(x-y)^(Length[suwir[[a]]])*Sum[Sum[FunctionExpand[FactorialPower[y,Length[IndependentPartitionsOfPartitionsOf[InduceSubgraph[graph,Complement[VertexList[graph],suwir[[a]]]],m,b][[c]]]]],{c,1,Length[IndependentPartitionsOfPartitionsOf[InduceSubgraph[graph,Complement[VertexList[graph],suwir[[a]]]],m,b]]}],{b,1,Length[PartitionsOf[InduceSubgraph[graph,Complement[VertexList[graph],suwir[[a]]]],m]]}]
,{a,1,Length[suwir]}]]

Clear[PartitionsOfSingletons]
PartitionsOfSingletons[graph_,m_,a_]:=
Module[{i,n=Length[PartitionsOf[graph,m][[a]]],conspar=PartitionsOf[graph,m][[a]],Singletons={}},
For[i=1,i<=n,i++,
If[Length[conspar[[i]]]==1,AppendTo[Singletons,conspar[[i]]]];];
Singletons]

Clear[IndependentPartitionsOfPartitionsOfWithoutA]
IndependentPartitionsOfPartitionsOfWithoutA[graph_,m_,a_,listlist_]:=
Module[{i,j,n=Length[SetPartitions[Complement[PartitionsOf[graph,m][[a]],listlist]]],part=SetPartitions[Complement[PartitionsOf[graph,m][[a]],listlist]],IndependentPartitions={}},
For[i=1, i<=n,i++,
append=True;
For[j=1,j<= Length[part[[i]]],j++,
If[Length[part[[i]][[j]]]!= Length[Combinatorica`ConnectedComponents[InduceSubgraph[graph,Flatten[part[[i]][[j]]]]]],append=False;Break[]];];
If[append,AppendTo[IndependentPartitions,part[[i]]];];];
IndependentPartitions]

Clear[Koeffizient2]
Koeffizient2[graph_,m_,x_,y_]:=
Sum[Sum[(x-y)^(Length[Subsets[PartitionsOfSingletons[graph,m,i]][[j]]])*Sum[FunctionExpand[FactorialPower[y,Length[IndependentPartitionsOfPartitionsOfWithoutA[graph,m,i,Subsets[PartitionsOfSingletons[graph,m,i]][[j]]][[k]]]]]
,{k,1,Length[IndependentPartitionsOfPartitionsOfWithoutA[graph,m,i,Subsets[PartitionsOfSingletons[graph,m,i]][[j]]]]}]
,{j,1,Length[Subsets[PartitionsOfSingletons[graph,m,i]]]}]
,{i,1,Length[PartitionsOf[graph,m]]}]

Clear[TrivariaterKoeffizientAzyklisch];
TrivariaterKoeffizientAzyklisch[graph_,m_,x_,y_]:=0/;m>Length[Edges[graph]]&&AcyclicQ[graph]
TrivariaterKoeffizientAzyklisch[graph_,m_,x_,y_]:=BivariatesPolynom1[graph,x, y]/;m==0&&AcyclicQ[graph]
TrivariaterKoeffizientAzyklisch[graph_,m_,x_,y_]:=Module[{e=Edges[graph],e1,e2,g1,g2,g3},
g1=DeleteEdge[graph,e[[1]]];
g2=MakeSimple[Contract[graph,e[[1]]]];
e1=Min[e[[1]]];
e2=Max[e[[1]]];
g3=DeleteVertices[graph,{e1,e2}];
Return[Expand[TrivariaterKoeffizientAzyklisch[g2,m-1,x,y]-(x-y)*TrivariaterKoeffizientAzyklisch[g3,m-1,x,y]+TrivariaterKoeffizientAzyklisch[g1,m,x,y]-TrivariaterKoeffizientAzyklisch[g2,m,x,y]+(x-y)*TrivariaterKoeffizientAzyklisch[g3,m,x,y]]];
]/;0<Length[Edges[graph]]&&0<m&&m<= Length[Edges[graph]]&&AcyclicQ[graph]
TrivariaterKoeffizientAzyklisch[graph_,m_,x_,y_]:="Keine L\[ODoubleDot]sung"/;Not[AcyclicGraphQ[graph]] 

Clear[EdgesWithLeafs];
EdgesWithLeafs[graph_]:=
Module[{i,j,e=Edges[graph],m=Length[Edges[graph]],WithLeaf={}},
For[i=1,i<=m,i++,
append=True;
For[j=1,j<= 1,j++,
If[Length[Neighborhood[graph,e[[i]][[j]],1]]-1>1&&Length[Neighborhood[graph,e[[i]][[j+1]],1]]-1>1,append=False];];
If[append,AppendTo[WithLeaf,e[[i]]]];];
WithLeaf]

Clear[TrivariaterKoeffizientWald];
TrivariaterKoeffizientWald[graph_,m_,x_,y_]:=0/;m>Length[Edges[graph]]&&AcyclicQ[graph]
TrivariaterKoeffizientWald[graph_,m_,x_,y_]:=BivariatesPolynom1[graph,x, y]/;m==0&&AcyclicQ[graph]
TrivariaterKoeffizientWald[graph_,m_,x_,y_]:=Module[{e=EdgesWithLeafs[graph],e1,e2,g1,g2,g3},
g1=DeleteEdge[graph,e[[1]]];
g2=MakeSimple[Contract[graph,e[[1]]]];
e1=Min[e[[1]]];
e2=Max[e[[1]]];
g3=DeleteVertices[graph,{e1,e2}];
Return[Expand[TrivariaterKoeffizientWald[g2,m-1,x,y]-(x-y)*TrivariaterKoeffizientWald[g3,m-1,x,y]+(x-1)*TrivariaterKoeffizientWald[g2,m,x,y]+(x-y)*TrivariaterKoeffizientWald[g3,m,x,y]]];
]/;0<Length[Edges[graph]]&&0<m&&m<= Length[Edges[graph]]&&AcyclicQ[graph]
TrivariaterKoeffizientWald[graph_,m_,x_,y_]:="Keine L\[ODoubleDot]sung"/;Not[AcyclicGraphQ[graph]] 

Clear[TrivariatesPolynom1];
TrivariatesPolynom1[graph_,x_,y_,z_]:=Sum[Koeffizient1[graph,i,x,y]*z^i,{i,0,Length[Edges[graph]]}]

Clear[TrivariatesPolynom2];
TrivariatesPolynom2[graph_,x_,y_,z_]:=Sum[Koeffizient2[graph,i,x,y]*z^i,{i,0,Length[Edges[graph]]}]

Clear[TrivariatesPolynomAzyklisch];
TrivariatesPolynomAzyklisch[graph_,x_,y_,z_]:=Sum[TrivariaterKoeffizientAzyklisch[graph,i,x,y]*z^i,{i,0,Length[Edges[graph]]}]

Clear[TrivariatesPolynomWald];
TrivariatesPolynomWald[graph_,x_,y_,z_]:=Sum[TrivariaterKoeffizientWald[graph,i,x,y]*z^i,{i,0,Length[Edges[graph]]}]


